// 3DGraphic.cpp: implementation of the C3DGraphic class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "3DGraphic.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

C3DGraphic::C3DGraphic():
	m_flXMin(MIN_XRANGE_DEFAULT), 
	m_flXMax(MAX_XRANGE_DEFAULT), 
	m_flYMin(MIN_YRANGE_DEFAULT), 
	m_flYMax(MAX_YRANGE_DEFAULT),
	m_hwndRenderTarget(NULL),
	m_pFunction(NULL),
	m_dwMaxVertexes(MAX_VERTEXES),
	m_p3DDevice(NULL),
	m_pDataVB(NULL),
	m_dwElementsInVB(NULL),
	m_vecEyeAt(10.0f, 10.0f, 10.0f),
	m_vecEyeTo(0.0f, 0.0f, 0.0f)
	
{
}
	
C3DGraphic::~C3DGraphic()
{
	
	if (m_pDataVB) {
		m_pDataVB->Release();
		m_pDataVB = NULL;
	}
	if (m_p3DDevice) {
		m_p3DDevice->Release();
		m_p3DDevice = NULL;
	}
}

HRESULT C3DGraphic::Create(CD3D8Application* p_p3DApplication, 
						   HWND p_hwndTarget, C3DFunction* p_pFunction)
{
	ATLASSERT(p_hwndTarget);
	ATLASSERT(p_pFunction);
	ATLASSERT(p_p3DApplication);
	
	HRESULT hr; 
	hr = SetFunction(p_pFunction);
	if (FAILED(hr)) {
		return hr;
	}
	
	hr = InitMaterial();
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	hr = InitLight();
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	m_dwBackColor = 0x00000000;

	m_hwndRenderTarget = p_hwndTarget;
	m_p3DApplication = p_p3DApplication;


	D3DDISPLAYMODE theDisplayMode;
	hr = m_p3DApplication->m_pDirect3DObject->GetAdapterDisplayMode(
		D3DADAPTER_DEFAULT, &theDisplayMode);
	if (FAILED(hr)) {
		return hr;
	}


	D3DPRESENT_PARAMETERS thePresentParams; 
	ZeroMemory(&thePresentParams, sizeof(thePresentParams));
	thePresentParams.Windowed   = TRUE;
	thePresentParams.SwapEffect = D3DSWAPEFFECT_COPY;
	thePresentParams.BackBufferFormat = theDisplayMode.Format;
	thePresentParams.EnableAutoDepthStencil = TRUE;
    thePresentParams.AutoDepthStencilFormat = D3DFMT_D16;
	
	hr = m_p3DApplication->m_pDirect3DObject->CreateDevice(
		D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
		m_hwndRenderTarget,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&thePresentParams,
		&m_p3DDevice);

	if (FAILED(hr)) {
		ATLTRACE(_T("C3DGraphic::Create >>>> CreateDevice() failed with\
hr = 0x%08x\n"), hr);
		return hr;
	}

	return S_OK;
}

HRESULT C3DGraphic::SetFunction(C3DFunction* p_pFunction)
{
	HRESULT hr = E_FAIL;
	ATLASSERT(p_pFunction);
	// Set new function;
	m_pFunction = p_pFunction;
	
	FLOAT flXMin, flXMax;
	FLOAT flYMin, flYMax;
	
	hr = m_pFunction->GetXRange(&flXMin, &flXMax);
	if (FAILED(hr)) {
		return hr;
	}

	if (S_FALSE == hr) {
		flXMin = MIN_XRANGE_DEFAULT;
		flXMax = MAX_XRANGE_DEFAULT;
	}

	hr = m_pFunction->GetYRange(&flYMin, &flYMax);
	if (FAILED(hr)) {
		return hr;
	}
	if (S_FALSE == hr) {
		flYMin = MIN_XRANGE_DEFAULT;
		flYMax = MAX_XRANGE_DEFAULT;
	}
	hr = SetXMin(flXMin);
	if (FAILED(hr)) {
		return hr;
	}
	hr = SetXMax(flXMax);
	if (FAILED(hr)) {
		return hr;
	}
	hr = SetYMin(flYMin);
	if (FAILED(hr)) {
		return hr;
	}
	hr = SetYMax(flYMax);
	if (FAILED(hr)) {
		return hr;
	}
	
	return S_OK;
}

HRESULT C3DGraphic::SetXMin(FLOAT p_flXMin)
{
	m_flXMin = p_flXMin;
	return S_OK;
}

HRESULT C3DGraphic::SetXMax(FLOAT p_flXMax)
{
	m_flXMax = p_flXMax;
	return S_OK;
}

HRESULT C3DGraphic::SetYMin(FLOAT p_flYMin)
{
	m_flYMin = p_flYMin;
	return S_OK;
}

HRESULT C3DGraphic::SetYMax(FLOAT p_flYMax)
{
	m_flYMax = p_flYMax;
	return S_OK;
}

HRESULT C3DGraphic::ReRender(BOOL p_bRecalcData)
{
	if (!m_p3DDevice) {
		return E_FAIL;
	}

	HRESULT hr = E_FAIL;


	if (p_bRecalcData) {
		hr = RecalculateData();
		if (FAILED(hr)) {
			return hr;
		}
	}

	hr = m_p3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
		m_dwBackColor, 1.0f, 0);
	if (FAILED(hr)) {
		return hr;
	}
	
	D3DXMATRIX mtxWorld;
	D3DXMATRIX mtxView;
	D3DXMATRIX mtxProjection;

	D3DXMatrixRotationX(&mtxWorld, D3DXToRadian(0));
	m_p3DDevice->SetTransform(D3DTS_WORLD, &mtxWorld);

	D3DXMatrixLookAtLH(&mtxView,
		&m_vecEyeAt,
		&m_vecEyeTo,
		&D3DXVECTOR3(0.0f, 0.0f, 1.0f));

	m_p3DDevice->SetTransform(D3DTS_VIEW, &mtxView);
	
	D3DXMatrixPerspectiveFovLH(&mtxProjection, D3DXToRadian(45), 1, 1.0f, 100.0f);
	hr = m_p3DDevice->SetTransform(D3DTS_PROJECTION, &mtxProjection);
	if (FAILED(hr)) {
		return hr;
	}

	hr = m_p3DDevice->BeginScene();
	if (FAILED(hr)) {
		return hr;
	}


	hr = m_p3DDevice->SetMaterial(&m_theGraphMaterial);
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}
	
	hr = m_p3DDevice->SetLight(0, &m_theGraphLight);
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	hr = m_p3DDevice->LightEnable(0, TRUE);
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	hr = m_p3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	hr = m_p3DDevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}
	
	hr = m_p3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_p3DDevice->SetStreamSource(0, m_pDataVB, sizeof(GRAPH3DVERTEXSTRUCT));
	if (FAILED(hr)) {
		return hr;
	}

	hr = m_p3DDevice->SetVertexShader(D3DFVF_GRAPH3DVERTEX);
	if (FAILED(hr)) {
		return hr;
	}
	
	hr = m_p3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, m_dwElementsInVB - 2);
	if (FAILED(hr)) {
		return hr;
	}

	
	hr = m_p3DDevice->EndScene();
	if (FAILED(hr)) {
		return hr;
	}
	
	return S_OK;
}

HRESULT C3DGraphic::BlitToWindow()
{
	if (!m_p3DDevice) {
		return E_FAIL;
	}

	HRESULT hr;
	hr = m_p3DDevice->Present(NULL, NULL, NULL, NULL);
	if (FAILED(hr)) {
		return hr;
	}
	return S_OK;
}

HRESULT C3DGraphic::CreateVertexBuffers(DWORD p_dwVertexesNum)
{
	if (p_dwVertexesNum == m_dwElementsInVB) {
		// VB already contains needed number of vertexes.
		// Reformatting is not needed.
		return S_OK;
	}

	HRESULT hr;
	if (m_pDataVB) {
		m_pDataVB->Release();
		m_pDataVB = NULL;
	}
	
	m_dwElementsInVB = 0;
	hr = m_p3DDevice->CreateVertexBuffer(
		sizeof(GRAPH3DVERTEXSTRUCT) * p_dwVertexesNum, 
		0, D3DFVF_GRAPH3DVERTEX,
		D3DPOOL_DEFAULT, &m_pDataVB);

	if (FAILED(hr)) {
		m_dwElementsInVB = 0;
		return hr;
	}

	m_dwElementsInVB = p_dwVertexesNum;
	return S_OK;
}

HRESULT C3DGraphic::RecalculateData()
{
	HRESULT hr = E_FAIL;

	CGraphGrid theGrid(GetFunction(), GetXMin(), GetXMax(), GetYMin(), GetYMax());
	hr = theGrid.CalcGridFromMaxPoints(m_dwMaxVertexes);
	ATLASSERT(SUCCEEDED(hr));

	DWORD dwPointsPerXAxe = theGrid.GetPointsPerX();
	DWORD dwPointsPerYAxe = theGrid.GetPointsPerY();

	DWORD dwVertexesNumber = (dwPointsPerYAxe - 1) * dwPointsPerXAxe * 2;
	
	hr = CreateVertexBuffers(dwVertexesNumber);
	if (FAILED(hr)) {
		return hr;
	}

	GRAPH3DVERTEXSTRUCT* pData;
	hr = m_pDataVB->Lock(0, 0, (PBYTE*)&pData, 0);
	if (FAILED(hr)) {
		return hr;
	}
	GRAPH3DVERTEXSTRUCT* pCurr = pData;
	
	DWORD dwCurrXIndex = 0;
	for (INT j = 0; j < theGrid.GetPointsPerY() - 1; j++) {
		for (INT i = 0; i < theGrid.GetPointsPerX(); i++) {
			if (0 == (j & 0x00000001)) {
				// even index
				dwCurrXIndex = i;

				pCurr->x = (theGrid.GetXAxeData())[dwCurrXIndex];
				pCurr->y = (theGrid.GetYAxeData())[j + 1];
				pCurr->z = (theGrid.GetGridData())[dwCurrXIndex + (j + 1) * 
					theGrid.GetPointsPerX()];
				hr = CalcNormal(&theGrid, dwCurrXIndex, j + 1, pCurr);
				ATLASSERT(SUCCEEDED(hr));
				
				pCurr++;
				
				pCurr->x = (theGrid.GetXAxeData())[dwCurrXIndex];
				pCurr->y = (theGrid.GetYAxeData())[j];
				pCurr->z = (theGrid.GetGridData())[dwCurrXIndex + j * 
					theGrid.GetPointsPerX()];
				hr = CalcNormal(&theGrid, dwCurrXIndex, j, pCurr);
				ATLASSERT(SUCCEEDED(hr));
				
				pCurr++;
				
				
			} else {
				// odd index
				dwCurrXIndex = theGrid.GetPointsPerX() - i - 1;
				
				pCurr->x = (theGrid.GetXAxeData())[dwCurrXIndex];
				pCurr->y = (theGrid.GetYAxeData())[j];
				pCurr->z = (theGrid.GetGridData())[dwCurrXIndex + (j) * 
					theGrid.GetPointsPerX()];
				hr = CalcNormal(&theGrid, dwCurrXIndex, j, pCurr);
				ATLASSERT(SUCCEEDED(hr));
				
				pCurr++;
				
				pCurr->x = (theGrid.GetXAxeData())[dwCurrXIndex];
				pCurr->y = (theGrid.GetYAxeData())[j + 1];
				pCurr->z = (theGrid.GetGridData())[dwCurrXIndex + (j + 1) * 
					theGrid.GetPointsPerX()];
				hr = CalcNormal(&theGrid, dwCurrXIndex, j + 1, pCurr);
				ATLASSERT(SUCCEEDED(hr));
				
				pCurr++;
				
			}
			

			
		}
	}

	hr = m_pDataVB->Unlock();
	if (FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

HRESULT C3DGraphic::SetEyeAt(FLOAT p_flX, FLOAT p_flY, FLOAT p_flZ, 
							 BOOL p_bRedraw)
{
	HRESULT hr;
	m_vecEyeAt.x = p_flX;
	m_vecEyeAt.y = p_flY;
	m_vecEyeAt.z = p_flZ;

	if (p_bRedraw) {
		hr = ReRender();
		ATLASSERT(SUCCEEDED(hr));
		if (FAILED(hr)) {
			return hr;
		}
		hr = BlitToWindow();
		ATLASSERT(SUCCEEDED(hr));
		if (FAILED(hr)) {
			return hr;
		}
	}
	
	return S_OK;
}

HRESULT C3DGraphic::SetEyeTo(FLOAT p_flX, FLOAT p_flY, FLOAT p_flZ, 
							 BOOL p_bRedraw)
{
	HRESULT hr;
	m_vecEyeTo.x = p_flX;
	m_vecEyeTo.y = p_flY;
	m_vecEyeTo.z = p_flZ;

	if (p_bRedraw) {
		hr = ReRender();
		ATLASSERT(SUCCEEDED(hr));
		if (FAILED(hr)) {
			return hr;
		}
		hr = BlitToWindow();
		ATLASSERT(SUCCEEDED(hr));
		if (FAILED(hr)) {
			return hr;
		}
	}
	return S_OK;
}

HRESULT C3DGraphic::CalcNormal(CGraphGrid* p_pGrid, DWORD p_dwX, DWORD p_dwY, 
				   GRAPH3DVERTEXSTRUCT* p_pVertex)
{
	INT nMaxXIndex = p_pGrid->GetPointsPerX() - 1;
	INT nMaxYIndex = p_pGrid->GetPointsPerY() - 1;
	
	INT nPoint1IndexX = p_dwX - 1;
	INT nPoint1IndexY = p_dwY;
	INT nPoint2IndexX = p_dwX;
	INT nPoint2IndexY = p_dwY + 1;
	INT nPoint3IndexX = p_dwX + 1;
	INT nPoint3IndexY = p_dwY;
	INT nPoint4IndexX = p_dwX;
	INT nPoint4IndexY = p_dwY - 1;
	
	D3DXVECTOR3 vecTargetPoint(
		(p_pGrid->GetXAxeData())[p_dwX], 
		(p_pGrid->GetYAxeData())[p_dwY],
		(p_pGrid->GetGridData())[p_dwX + p_dwY * p_pGrid->GetPointsPerX()]); 

	D3DXVECTOR3 vecVector1;
	D3DXVECTOR3 vecVector2;
	D3DXVECTOR3 vecVector3;
	D3DXVECTOR3 vecVector4;
	
	if (nPoint1IndexX < 0 || nPoint1IndexX > nMaxXIndex ||
		nPoint1IndexY < 0 || nPoint1IndexY > nMaxYIndex) {
		// Point is out of grid. Fill vector with simple values...
		vecVector1.x = -1.0f;
		vecVector1.y = 0.0f;
		vecVector1.z = 0.0f;
	} else {
		// POINT is inside the grid. Calculate vector...
		D3DXVECTOR3 vecTempPoint(
			(p_pGrid->GetXAxeData())[nPoint1IndexX], 
			(p_pGrid->GetYAxeData())[nPoint1IndexY],
			(p_pGrid->GetGridData())[nPoint1IndexX + nPoint1IndexY * 
				p_pGrid->GetPointsPerX()]);
		vecVector1 = vecTempPoint - vecTargetPoint;
		
	}

	if (nPoint2IndexX < 0 || nPoint2IndexX > nMaxXIndex ||
		nPoint2IndexY < 0 || nPoint2IndexY > nMaxYIndex) {
		// Point is out of grid. Fill vector with simple values...
		vecVector2.x = 0.0f;
		vecVector2.y = +1.0f;
		vecVector2.z = 0.0f;
	} else {
		// POINT is inside the grid. Calculate vector...
		D3DXVECTOR3 vecTempPoint(
			(p_pGrid->GetXAxeData())[nPoint2IndexX], 
			(p_pGrid->GetYAxeData())[nPoint2IndexY],
			(p_pGrid->GetGridData())[nPoint2IndexX + nPoint2IndexY * 
			p_pGrid->GetPointsPerX()]);
		vecVector2 = vecTempPoint - vecTargetPoint;
		
	}

	if (nPoint3IndexX < 0 || nPoint3IndexX > nMaxXIndex ||
		nPoint3IndexY < 0 || nPoint3IndexY > nMaxYIndex) {
		// Point is out of grid. Fill vector with simple values...
		vecVector3.x = +1.0f;
		vecVector3.y = 0.0f;
		vecVector3.z = 0.0f;
	} else {
		// POINT is inside the grid. Calculate vector...
		D3DXVECTOR3 vecTempPoint(
			(p_pGrid->GetXAxeData())[nPoint3IndexX], 
			(p_pGrid->GetYAxeData())[nPoint3IndexY],
			(p_pGrid->GetGridData())[nPoint3IndexX + nPoint3IndexY * 
			p_pGrid->GetPointsPerX()]);
		vecVector3 = vecTempPoint - vecTargetPoint;
		
	}

	if (nPoint4IndexX < 0 || nPoint4IndexX > nMaxXIndex ||
		nPoint4IndexY < 0 || nPoint4IndexY > nMaxYIndex) {
		// Point is out of grid. Fill vector with simple values...
		vecVector4.x = 0.0f;
		vecVector4.y = -1.0f;
		vecVector4.z = 0.0f;
	} else {
		// POINT is inside the grid. Calculate vector...
		D3DXVECTOR3 vecTempPoint(
			(p_pGrid->GetXAxeData())[nPoint4IndexX], 
			(p_pGrid->GetYAxeData())[nPoint4IndexY],
			(p_pGrid->GetGridData())[nPoint4IndexX + nPoint4IndexY * 
			p_pGrid->GetPointsPerX()]);
		vecVector4 = vecTempPoint - vecTargetPoint;
		
	}
	
	D3DXVECTOR3 vecCrossProduct1;
	D3DXVECTOR3 vecCrossProduct2;
	D3DXVECTOR3 vecCrossProduct3;
	D3DXVECTOR3 vecCrossProduct4;

	// Calculate four normals to each plane...
	D3DXVec3Cross(&vecCrossProduct1, &vecVector2, &vecVector1);
	D3DXVec3Cross(&vecCrossProduct2, &vecVector3, &vecVector2);
	D3DXVec3Cross(&vecCrossProduct3, &vecVector4, &vecVector3);
	D3DXVec3Cross(&vecCrossProduct4, &vecVector1, &vecVector4);

	// Normalize normals...
	D3DXVec3Normalize(&vecCrossProduct1, &vecCrossProduct1);
	D3DXVec3Normalize(&vecCrossProduct2, &vecCrossProduct2);
	D3DXVec3Normalize(&vecCrossProduct3, &vecCrossProduct3);
	D3DXVec3Normalize(&vecCrossProduct4, &vecCrossProduct4);
	
	// Finding average normal...
	D3DXVECTOR3 vecNormal = (vecCrossProduct1 +
		vecCrossProduct2 +
		vecCrossProduct3 +
		vecCrossProduct4) / 4.0f;

	p_pVertex->nx = vecNormal.x;
	p_pVertex->ny = vecNormal.y;
	p_pVertex->nz = vecNormal.z;
	
	return S_OK;
}


HRESULT C3DGraphic::InitLight()
{
	HRESULT hr = E_FAIL;
	ZeroMemory(&m_theGraphLight, sizeof(m_theGraphLight));
	
	// Setting up light parameters
	m_theGraphLight.Type = D3DLIGHT_DIRECTIONAL;

	m_theGraphLight.Direction = D3DXVECTOR3(0.0f, 0.0f, -1.0f);
	m_theGraphLight.Ambient.r = 0.0f;
	m_theGraphLight.Ambient.g = 0.3f;
	m_theGraphLight.Ambient.b = 0.0f;
	m_theGraphLight.Ambient.a = 0.0f;
	
	m_theGraphLight.Diffuse.r = 1.0f;
	m_theGraphLight.Diffuse.g = 1.0f;
	m_theGraphLight.Diffuse.b = 1.0f;
	m_theGraphLight.Diffuse.a = 1.0f;

	m_theGraphLight.Specular.r = 1.0f;
	m_theGraphLight.Specular.g = 1.0f;
	m_theGraphLight.Specular.b = 1.0f;
	m_theGraphLight.Specular.a = 1.0f;
	
	return S_OK;
}

HRESULT C3DGraphic::InitMaterial()
{
	ZeroMemory(&m_theGraphMaterial, sizeof(m_theGraphMaterial));
	m_theGraphMaterial.Ambient.r = 0.0f;
	m_theGraphMaterial.Ambient.g = 0.0f;
	m_theGraphMaterial.Ambient.b = 0.0f;
	m_theGraphMaterial.Ambient.a = 0.0f;
	
	m_theGraphMaterial.Diffuse.r = 0.5f;
	m_theGraphMaterial.Diffuse.g = 0.5f;
	m_theGraphMaterial.Diffuse.b = 0.5f;
	m_theGraphMaterial.Diffuse.a = 0.5f;
	
	m_theGraphMaterial.Specular.r = 0.7f;
	m_theGraphMaterial.Specular.g = 0.7f;
	m_theGraphMaterial.Specular.b = 0.7f;
	m_theGraphMaterial.Specular.a = 0.7f;
	m_theGraphMaterial.Power = 0.5;
	
	m_theGraphMaterial.Emissive.r = 0.0f;
	m_theGraphMaterial.Emissive.g = 0.0f;
	m_theGraphMaterial.Emissive.b = 0.0f;
	m_theGraphMaterial.Emissive.a = 0.0f;
	


	return S_OK;
}

HRESULT C3DGraphic::SetMaterialAmbient(FLOAT p_flR, 
									   FLOAT p_flG, 
									   FLOAT p_flB)
{
	m_theGraphMaterial.Ambient.r = p_flR;
	m_theGraphMaterial.Ambient.g = p_flG;
	m_theGraphMaterial.Ambient.b = p_flB;
	m_theGraphMaterial.Ambient.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetMaterialDiffuse(FLOAT p_flR, 
									   FLOAT p_flG, 
									   FLOAT p_flB)
{
	m_theGraphMaterial.Diffuse.r = p_flR;
	m_theGraphMaterial.Diffuse.g = p_flG;
	m_theGraphMaterial.Diffuse.b = p_flB;
	m_theGraphMaterial.Diffuse.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetMaterialEmissive(FLOAT p_flR, 
										FLOAT p_flG, 
										FLOAT p_flB)
{
	m_theGraphMaterial.Emissive.r = p_flR;
	m_theGraphMaterial.Emissive.g = p_flG;
	m_theGraphMaterial.Emissive.b = p_flB;
	m_theGraphMaterial.Emissive.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetMaterialSpecular(FLOAT p_flR, 
										FLOAT p_flG, 
										FLOAT p_flB, 
										FLOAT p_flP)
{
	m_theGraphMaterial.Specular.r = p_flR;
	m_theGraphMaterial.Specular.g = p_flG;
	m_theGraphMaterial.Specular.b = p_flB;
	m_theGraphMaterial.Specular.a = 0.0f;
	m_theGraphMaterial.Power = p_flP;

	return S_OK;
}

HRESULT C3DGraphic::SetLightDirection(FLOAT p_flX, 
									  FLOAT p_flY, 
									  FLOAT p_flZ)
{
	m_theGraphLight.Direction.x = p_flX;
	m_theGraphLight.Direction.y = p_flY;
	m_theGraphLight.Direction.z = p_flZ;
	return S_OK;
}

HRESULT C3DGraphic::GetLightDirection(PFLOAT p_pflX, 
									  PFLOAT p_pflY, 
									  PFLOAT p_pflZ)
{
	*p_pflX = m_theGraphLight.Direction.x;
	*p_pflY = m_theGraphLight.Direction.y;
	*p_pflZ = m_theGraphLight.Direction.z;
	return S_OK;
}

HRESULT C3DGraphic::SetLightAmbient(FLOAT p_flR, 
									FLOAT p_flG, 
									FLOAT p_flB)
{
	m_theGraphLight.Ambient.r = p_flR;
	m_theGraphLight.Ambient.g = p_flG;
	m_theGraphLight.Ambient.b = p_flB;
	m_theGraphLight.Ambient.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetLightDiffuse(FLOAT p_flR, 
									FLOAT p_flG, 
									FLOAT p_flB)
{
	m_theGraphLight.Diffuse.r = p_flR;
	m_theGraphLight.Diffuse.g = p_flG;
	m_theGraphLight.Diffuse.b = p_flB;
	m_theGraphLight.Diffuse.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetLightSpecular(FLOAT p_flR, 
									 FLOAT p_flG, 
									 FLOAT p_flB)
{
	m_theGraphLight.Specular.r = p_flR;
	m_theGraphLight.Specular.g = p_flG;
	m_theGraphLight.Specular.b = p_flB;
	m_theGraphLight.Specular.a = 0.0f;
	return S_OK;
}

HRESULT C3DGraphic::SetBackColor(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB)
{
	if (p_flR < 0.0f) p_flR = 0.0f;
	if (p_flG < 0.0f) p_flG = 0.0f;
	if (p_flB < 0.0f) p_flB = 0.0f;
	if (p_flR > 1.0f) p_flR = 1.0f;
	if (p_flG > 1.0f) p_flG = 1.0f;
	if (p_flB > 1.0f) p_flB = 1.0f;

	m_dwBackColor = D3DCOLOR_XRGB(DWORD(p_flR * 0xFF), 
		DWORD(p_flG * 0xFF), DWORD(p_flB * 0xFF));

	return S_OK;
}

HRESULT C3DGraphic::GetBackColor(PFLOAT p_pflR, PFLOAT p_pflG, PFLOAT p_pflB)
{
	
	return E_NOTIMPL;
}
